@@ -4,12 +4,10 @@
 from dulwich.client import Urllib3HttpGitClient
 from dulwich.config import StackedConfig
 
-from .credentials import CredentialNotFoundError, get_credentials_from_helper
+from scmrepo.git.credentials import CredentialNotFoundError, get_credentials_from_helper
 
 
-class GitCredentialsHTTPClient(
-    Urllib3HttpGitClient
-):  # pylint: disable=abstract-method
+class GitCredentialsHTTPClient(Urllib3HttpGitClient):  # pylint: disable=abstract-method
     def __init__(
         self,
         base_url,
