@@ -7,6 +7,8 @@ from io import BytesIO, StringIO
 from typing import (
     TYPE_CHECKING,
     Callable,
+    Dict,
+    Generator,
     Iterable,
     List,
     Mapping,
@@ -15,14 +17,10 @@ from typing import (
     Union,
 )
 
-from funcy import cached_property, first
+from funcy import cached_property, reraise
+from shortuuid import uuid
 
-from scmrepo.exceptions import (
-    CloneError,
-    MergeConflictError,
-    RevError,
-    SCMError,
-)
+from scmrepo.exceptions import CloneError, MergeConflictError, RevError, SCMError
 from scmrepo.utils import relpath
 
 from ..objects import GitCommit, GitObject
@@ -32,7 +30,7 @@ logger = logging.getLogger(__name__)
 
 
 if TYPE_CHECKING:
-    from pygit2 import Signature
+    from pygit2.remote import Remote  # type: ignore
 
     from scmrepo.progress import GitProgressEvent
 
@@ -125,31 +123,11 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
         return commit, ref
 
     @property
-    def default_signature(self) -> "Signature":
+    def default_signature(self):
         try:
             return self.repo.default_signature
         except KeyError as exc:
-            signature = self._get_codespaces_signature()
-            if signature is not None:
-                return signature
-            raise SCMError(
-                "Git username and email must be configured"
-            ) from exc
-
-    def _get_codespaces_signature(self) -> Optional["Signature"]:
-        from pygit2 import Config, Signature
-
-        if "CODESPACES" not in os.environ:
-            return None
-        try:
-            config = Config("/usr/local/etc/gitconfig")
-            name = first(config.get_multivar("user.name"))
-            email = first(config.get_multivar("user.email"))
-            if name and email:
-                return Signature(name, email)
-        except Exception:  # nosec B110, pylint: disable=broad-except
-            pass
-        return None
+            raise SCMError("Git username and email must be configured") from exc
 
     @staticmethod
     def _get_checkout_strategy(strategy: Optional[int] = None):
@@ -217,9 +195,7 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
     ):
         from pygit2 import GIT_CHECKOUT_FORCE, GitError
 
-        strategy = self._get_checkout_strategy(
-            GIT_CHECKOUT_FORCE if force else None
-        )
+        strategy = self._get_checkout_strategy(GIT_CHECKOUT_FORCE if force else None)
 
         with self.release_odb_handles():
             if create_new:
@@ -359,21 +335,12 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
         if message:
             self._refdb.ensure_log(name)
         if symbolic:
-            self.repo.create_reference_symbolic(
-                name, new_ref, True, message=message
-            )
+            self.repo.create_reference_symbolic(name, new_ref, True, message=message)
         else:
-            self.repo.create_reference_direct(
-                name, new_ref, True, message=message
-            )
+            self.repo.create_reference_direct(name, new_ref, True, message=message)
 
     def get_ref(self, name, follow: bool = True) -> Optional[str]:
-        from pygit2 import (
-            GIT_OBJ_COMMIT,
-            GIT_REF_SYMBOLIC,
-            InvalidSpecError,
-            Tag,
-        )
+        from pygit2 import GIT_OBJ_COMMIT, GIT_REF_SYMBOLIC, InvalidSpecError, Tag
 
         try:
             ref = self.repo.references.get(name)
@@ -450,6 +417,60 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
     ) -> Mapping[str, SyncStatus]:
         raise NotImplementedError
 
+    def _merge_remote_branch(
+        self,
+        rh: str,
+        lh: str,
+        force: bool = False,
+        on_diverged: Optional[Callable[[str, str], bool]] = None,
+    ) -> SyncStatus:
+        import pygit2
+
+        rh_rev = self.resolve_rev(rh)
+
+        if force:
+            self.set_ref(lh, rh_rev)
+            return SyncStatus.SUCCESS
+
+        try:
+            merge_result, _ = self.repo.merge_analysis(rh_rev, lh)
+        except KeyError:
+            self.set_ref(lh, rh_rev)
+            return SyncStatus.SUCCESS
+
+        if merge_result & pygit2.GIT_MERGE_ANALYSIS_UP_TO_DATE:
+            return SyncStatus.UP_TO_DATE
+        if merge_result & pygit2.GIT_MERGE_ANALYSIS_FASTFORWARD:
+            self.set_ref(lh, rh_rev)
+            return SyncStatus.SUCCESS
+        if merge_result & pygit2.GIT_MERGE_ANALYSIS_NORMAL:
+            if on_diverged and on_diverged(lh, rh_rev):
+                return SyncStatus.SUCCESS
+            return SyncStatus.DIVERGED
+        logger.debug("Unexpected merge result: %s", pygit2.GIT_MERGE_ANALYSIS_NORMAL)
+        raise SCMError("Unknown merge analysis result")
+
+    @contextmanager
+    def get_remote(self, url: str) -> Generator["Remote", None, None]:
+        try:
+            remote = self.repo.remotes[url]
+            url = remote.url
+        except ValueError:
+            pass
+        except KeyError:
+            raise SCMError(f"'{url}' is not a valid Git remote or URL")
+
+        if os.name == "nt" and url.startswith("ssh://"):
+            raise NotImplementedError
+        if os.name == "nt" and url.startswith("file://"):
+            url = url[len("file://") :]
+
+        try:
+            remote_name = uuid()
+            yield self.repo.remotes.create(remote_name, url)
+        finally:
+            self.repo.remotes.delete(remote_name)
+
     def fetch_refspecs(
         self,
         url: str,
@@ -459,7 +480,50 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
         progress: Callable[["GitProgressEvent"], None] = None,
         **kwargs,
     ) -> Mapping[str, SyncStatus]:
-        raise NotImplementedError
+        from pygit2 import GitError
+
+        if isinstance(refspecs, str):
+            refspecs = [refspecs]
+
+        with self.get_remote(url) as remote:
+            fetch_refspecs: List[str] = []
+            for refspec in refspecs:
+                if ":" in refspec:
+                    lh, rh = refspec.split(":")
+                else:
+                    lh = rh = refspec
+                if not rh.startswith("refs/"):
+                    rh = f"refs/heads/{rh}"
+                if not lh.startswith("refs/"):
+                    lh = f"refs/heads/{lh}"
+                rh = rh[len("refs/") :]
+                refspec = f"+{lh}:refs/remotes/{remote.name}/{rh}"
+                fetch_refspecs.append(refspec)
+
+            logger.debug("fetch_refspecs: %s", fetch_refspecs)
+            with reraise(
+                GitError,
+                SCMError(f"Git failed to fetch ref from '{url}'"),
+            ):
+                remote.fetch(refspecs=fetch_refspecs)
+
+            result: Dict[str, "SyncStatus"] = {}
+            for refspec in fetch_refspecs:
+                _, rh = refspec.split(":")
+                if not rh.endswith("*"):
+                    refname = rh.split("/", 3)[-1]
+                    refname = f"refs/{refname}"
+                    result[refname] = self._merge_remote_branch(
+                        rh, refname, force, on_diverged
+                    )
+                    continue
+                rh = rh.rstrip("*").rstrip("/") + "/"
+                for branch in self.iter_refs(base=rh):
+                    refname = f"refs/{branch[len(rh):]}"
+                    result[refname] = self._merge_remote_branch(
+                        branch, refname, force, on_diverged
+                    )
+        return result
 
     def _stash_iter(self, ref: str):
         raise NotImplementedError
@@ -602,9 +666,7 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
             path_list = None
 
         with self.release_odb_handles():
-            self.repo.checkout_index(
-                index=index, paths=path_list, strategy=strategy
-            )
+            self.repo.checkout_index(index=index, paths=path_list, strategy=strategy)
 
             if index.conflicts and (ours or theirs):
                 for ancestor, ours_entry, theirs_entry in index.conflicts:
@@ -720,9 +782,7 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
                 raise MergeConflictError("Merge contained conflicts")
 
             try:
-                if not (
-                    squash or ff_pref & GIT_MERGE_PREFERENCE_NO_FASTFORWARD
-                ):
+                if not (squash or ff_pref & GIT_MERGE_PREFERENCE_NO_FASTFORWARD):
                     if analysis & GIT_MERGE_ANALYSIS_FASTFORWARD:
                         return self._merge_ff(rev, obj)
 
@@ -773,3 +833,6 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
 
     def validate_git_remote(self, url: str, **kwargs):
         raise NotImplementedError
+
+    def check_ref_format(self, refname: str):
+        raise NotImplementedError
